/**
  *  \file client/chartloc.h
  */
#ifndef PCC_V2_CLIENT_CHARTLOC_H
#define PCC_V2_CLIENT_CHARTLOC_H

#include "client/chart.h"
#include "game/objlist.h"

/** Starchart location. Manages a location and viewport for the
    starchart, and a redraw flag. */
class GChartLocation {
    GChartViewport viewport;    /* contains center */
    GPoint         location;    /* location */
    GObjectList    list;        /* list of objects at this location */
    int            pid;         /* planet Id at this location */
    int            list_index;

    bool changed;

    void updateLocation();
 public:
    GChartLocation();
    ~GChartLocation();

    void init();
    void initAfterLoad();
    void initEntry(const GfxRect& area);

    void      setCenter(GPoint pt);
    GPoint    getCenter() const;
    void      setLocation(GPoint pt);
    GPoint    getLocation() const;

    int       getCurrentPlanetId() const;
    int       getCurrentShipId() const;
    GObjectId getCurrentObject() const;
    void      setCurrentObject(GObject& obj);

    void      doZoom(bool enlarge);
    void      doScroll(int direction, bool mark);

    GChartViewport& getViewport();

    const GObjectList& getObjectList() const;

    void toggleOptions(int32 which);

    bool isChanged() const;
    void markChanged();
    void clearChanged();
};


inline GPoint
GChartLocation::getCenter() const
{
    return viewport.center;
}

inline GPoint
GChartLocation::getLocation() const
{
    return location;
}

inline GChartViewport&
GChartLocation::getViewport()
{
    // cannot be const, because the interesting methods of GChartViewport are not const
    return viewport;
}

inline const GObjectList&
GChartLocation::getObjectList() const
{
    return list;
}

inline int
GChartLocation::getCurrentPlanetId() const
{
    return pid;
}

#endif
