/**
  *  \file client/wscanchart.h
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WSCANCHART_H
#define PCC_V2_CLIENT_WSCANCHART_H

#include "ui/ui.h"
#include "client/wchart.h"
#include "gfx/gfx.h"
#include "game/coord.h"
#include "cpluslib/sigslot.h"
#include "game/lock.h"

class WScannerChartWidget : public WChartWidget {
    GPoint      scan;
    SignalConnection conn_change;
    // FIXME: bubble stuff
    void drawScannerXor(GfxCanvas& can);
    void moveKey(int dx, int dy);
 protected:
    virtual void drawScanner(GfxCanvas& can);

    virtual void  lockQueryLocation(bool& hyperjumping, int& current_gravity_pid);
    virtual int32 lockQueryDistance(GPoint pt);

    static void drawEndBig(GfxCanvas& can, GfxParameters& parm, GfxPoint pt);
    static void drawEndSmall(GfxCanvas& can, GfxParameters& parm, GfxPoint pt);

    bool handleKey(UIEvent::Key k);
 public:
    WScannerChartWidget();
    ~WScannerChartWidget();
    void drawPre(GfxCanvas& can);
    void drawPost(GfxCanvas& can);
    void setScannerPosition(GPoint pt);
    void setCenterPosition(GPoint pt);
    void doItemLock(GPoint clicked, TLockMode mode, bool marked_only, bool optimize_warp);
    bool handleEvent(const UIEvent& e, bool second_pass);

    void doKeyboardMode();

    GPoint getScannerPosition() const
        { return scan; }

    GPoint clipScannerPosition(GPoint pt) const;

    virtual void userMoveScannerTo(GPoint pt);

    void getLayoutInfo(LayoutInfo& info);

    enum {
        sc_StyleChange  = st_Last*2,  ///< user can changes style (C-M-letter)
        sc_Zoomable     = st_Last*4,  ///< user can zoom (+/-)
        sc_Keyboard     = st_Last*8,  ///< user can use keyboard
        sc_ShipsOnly    = st_Last*16, ///< only lock on ships
        sc_KeyboardMode = st_Last*32  ///< keyboard mode (Y) active
    };

    Signal1<int> sig_moved;
    Signal1<int> sig_dblclick;
};

#endif
