/**
  *  \file game/gen.h
  *  \brief "GEN" file management
  *
  *  This provides a class to parse GENx.DAT files and the respective
  *  RST section.
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_GEN_H
#define PCC_V2_GAME_GEN_H

#include "game/struct.h"
#include "cpluslib/string.h"
#include "io/io.h"
#include "game/timestamp.h"
#include "game/checksum.h"

//! GENx.DAT parser.
class GGen {
    TGen gen;
    char sig2[10];
    unsigned int getPassword(char (&p)[10]) const;
    void genSig();
 public:
    static const char NOPASSWORD[];

    GGen() { }
    GGen(const TGen& agen);
    GGen(const TResultGen& agen);

    /// Get turn number as contained in the file.
    int getTurnNumber() const
        { return gen.turn; }
    /// Get player number as contained in the file.
    int getPlayerId() const
        { return gen.player_id; }
    /// Get timestamp.
    GTimestamp getTimestamp() const
        { return GTimestamp(gen.timestamp); }

    /* Password access */
    bool isPassword(string_t pass) const;
    bool hasPassword() const;
    void setPassword(string_t pass);
    void setPasswordFromTurn(const char* data);

    /// Get signature 1 as per filefmt.txt (DIS signature).
    const char* getSig1() const
        { return &gen.password[10]; }
    /// Get signature 2 as per filefmt.txt (DAT signature).
    const char* getSig2() const
        { return sig2; }
    /// Get raw GEN record.
    const TGen& getGenRecord() const
        { return gen; }

    uint32 getChecksum(GControlSection sec) const;
    void setChecksum(GControlSection sec, uint32 value);

    bool isValidTimestampChecksum() const;
    void setTimestampChecksum();
};

#endif
