/**
  *  \file game/host.cc
  *  \brief Knowledge about Host Properties
  *
  *  Essentially, we have two families of hosts, that's Tim's Host and
  *  PHost. In addition, there are version dependencies (like: Tim's
  *  Host in version 3.22.031+ does not allow you to place high-tech
  *  torps on a low-tech base; or, PHost introduced death rays in
  *  version 4.0.). This module should contain all such host version
  *  dependencies.
  *
  *  As a rule, if the version dependency is "in X, the rule is this,
  *  and in Y, the rule is that", we provide just a function to
  *  distinguish the X and Y cases. In some cases, we also provide a
  *  more detailed function, like "what is the limit for Z in this
  *  host version?".
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/host.h"

/** Host information. Contains properties of current host used for
    playing. */
GHost host;

/** Finish loading. This will fill in information that is still
    missing.
    - for THost, guess version number from HConfig size. */
void
GHost::finish()
{
    if (!host_version) {
        if (isTHost()) {
            if (hconfig_size > 338)
                host_version = MKVERSION(3,22,36);
            else if (hconfig_size > 336)
                host_version = MKVERSION(3,22,22);
            else if (hconfig_size > 302)
                host_version = MKVERSION(3,20,0);
            else if (hconfig_size > 298)
                host_version = MKVERSION(3,13,2);
            else if (hconfig_size > 288)
                host_version = MKVERSION(3,12,3);
            else if (hconfig_size > 186)
                host_version = MKVERSION(3,11,1);
            else if (hconfig_size > 10)
                host_version = MKVERSION(3,10,0); // 3.10 (?)
            else
                host_version = MKVERSION(3,0,0);
        } else {
            host_version = MKVERSION(3,4,0);
        }
    }
}

/** Clear. Brings the GHost information back to its virgin state. */
void
GHost::clear()
{
    host_type    = ht_Unknown;
    host_version = 0;
    hconfig_size = 0;
}
