/**
  *  \file game/objnote.h
  *  \brief Note Objects
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_OBJNOTE_H
#define PCC_V2_GAME_OBJNOTE_H

#include "game/obj.h"
#include "game/objtypes.h"
#include "cpluslib/string.h"
#include "cpluslib/ptrvector.h"

/** Note Object. This is a pseudo object used to add additional entries to
    object lists. Each of those consists of an Id number and a name.
    If the Id number is zero, the object is called a <b>divider</b> and
    can be treated specially in various ways. */
class GNoteObject : public GObject {
    int id;
    string_t name;
 public:
    GNoteObject(int id, string_t name);
    GNoteObject(const GNoteObject& other);
    ~GNoteObject();

    string_t getName() const;
    string_t getFullName() const;
    int      getId() const;
    int      getOwner() const;
    bool     isExistant() const;
};

/** Note Container. Manages a set of GNoteObject's, and acts as GObjectType
    for them. */
class GNoteContainer : public GObjectType {
    ptr_vector<GNoteObject> notes;

 public:
    void         init();
    bool         isExistant(int index);
    GNoteObject& getObject(int index);
    int          getNext(int index, bool mark);
    int          getPrev(int index, bool mark);

    int          add(int id, string_t name);
    int          addCopyOf(const GNoteObject& obj);
    void         clear();
    void         swap(GNoteContainer& other);
};

#endif
