/**
  *  \file game/player.h
  *  \brief Player Properties and Accessors
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_PLAYER_H
#define PCC_V2_PLAYER_H

#include <string>
#include "cpluslib/smallset.h"
#include "cpluslib/string.h"
#include "io/streams.h"
#include "cpluslib/assert.h"

/** Player-to-Player Relation. */
enum GPlayerRelation {
    is_Me,                      ///< It's we.
    is_Ally,                    ///< It's an ally.
    is_Enemy                    ///< It's an enemy.
};

/** Set of players. */
typedef SmallSet<int> GPlayerSet;


class GPlayerBitMatrix {
    GPlayerSet data[12];
 public:
    GPlayerBitMatrix();

    bool get(int subj, int obj) const;
    void set(int subj, int obj, bool value);

    void clear();

    GPlayerSet getRow(int subj) const;
};

/** Array of player. */
template<typename T>
class GPlayerArray {
    T data[13];
 public:
    T& operator[](int pl)
        {
            ASSERT(isValidIndex(pl));
            return data[pl];
        }

    const T& operator[](int pl) const
        {
            ASSERT(isValidIndex(pl));
            return data[pl];
        }

    void setAll(int n);

    static bool isValidIndex(int pl)
        { return pl >= 0 && pl <= 12; }
};

/** Race name list. Routines to parse and query a race.nm file. */
class GRaceNameList {
    string_t short_names[13], long_names[13], adjectives[13];
 public:
    GRaceNameList();
    GRaceNameList(Stream& s);
    ~GRaceNameList();
    void             load(Stream&);
    const string_t&  getShortName(int player);
    const string_t&  getFullName(int player);
    const string_t&  getAdjName(int player);

    string_t         expandNames(string_t text);
};

extern GRaceNameList  racename_data;
extern const char     player_chars[];

inline string_t
getShortName(int player)
{
    return racename_data.getShortName(player);
}

inline string_t
getFullName(int player)
{
    return racename_data.getFullName(player);
}

inline string_t
getAdjName(int player)
{
    return racename_data.getAdjName(player);
}

inline string_t
expandRaceNames(const string_t& text)
{
    return racename_data.expandNames(text);
}

/* These player ids are used for display, not for rules */
int              getPlayerId();
int              getRealPlayerId();
void             setPlayerIds(int eff, int real);
GPlayerRelation  getPlayerRelation(int n);
int              getPlayerIdFromChar(char c);
GPlayerSet       parsePlayerList(const string_t& text, string_t::size_type& index);
GPlayerSet       parsePlayerSetFromNumberList(const string_t& text);
string_t         formatPlayerSetToNumberList(GPlayerSet set);
bool             allowRegistered(int player);
void             initRaceNames();

inline char getPlayerChar(int player)
{
    return player_chars[player];
}


template<typename T>
void
GPlayerArray<T>::setAll(int n)
{
    for (int i = 0; i < 13; ++i)
        data[i] = n;
}

#endif
