#include "game/player.h"
#include "u/t_game.h"
#include "u/files.h"
#include "io/memstream.h"

void
GamePlayerTestSuite::testParser()
{
    // getPlayerIdFromChar
    TS_ASSERT_EQUALS(getPlayerIdFromChar('0'), 0);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('1'), 1);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('2'), 2);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('3'), 3);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('4'), 4);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('5'), 5);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('6'), 6);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('7'), 7);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('8'), 8);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('9'), 9);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('a'), 10);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('A'), 10);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('b'), 11);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('B'), 11);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('c'), -1);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('C'), -1);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('Q'), -1);
    TS_ASSERT_EQUALS(getPlayerIdFromChar(' '), -1);
    TS_ASSERT_EQUALS(getPlayerIdFromChar('X'), -1);

    // getPlayerChar
    TS_ASSERT_EQUALS(getPlayerChar(0), '0');
    TS_ASSERT_EQUALS(getPlayerChar(1), '1');
    TS_ASSERT_EQUALS(getPlayerChar(2), '2');
    TS_ASSERT_EQUALS(getPlayerChar(3), '3');
    TS_ASSERT_EQUALS(getPlayerChar(4), '4');
    TS_ASSERT_EQUALS(getPlayerChar(5), '5');
    TS_ASSERT_EQUALS(getPlayerChar(6), '6');
    TS_ASSERT_EQUALS(getPlayerChar(7), '7');
    TS_ASSERT_EQUALS(getPlayerChar(8), '8');
    TS_ASSERT_EQUALS(getPlayerChar(9), '9');
    TS_ASSERT_EQUALS(getPlayerChar(10), 'A');
    TS_ASSERT_EQUALS(getPlayerChar(11), 'B');
    TS_ASSERT_EQUALS(getPlayerChar(12), 'X');

    // parsePlayerList: FIXME: routine is pending-delete

    // parsePlayerSetFromNumberList()
    TS_ASSERT_EQUALS(parsePlayerSetFromNumberList("").toInteger(), 0);
    TS_ASSERT_EQUALS(parsePlayerSetFromNumberList("1").toInteger(), (1 << 1));
    TS_ASSERT_EQUALS(parsePlayerSetFromNumberList("11").toInteger(), (1 << 11));
    TS_ASSERT_EQUALS(parsePlayerSetFromNumberList("1 2").toInteger(), (1 << 1) | (1 << 2));
    TS_ASSERT_EQUALS(parsePlayerSetFromNumberList("a").toInteger(), 0);

    // formatPlayerSetToNumberList()
    TS_ASSERT_EQUALS(formatPlayerSetToNumberList(GPlayerSet::fromInteger(1 << 11)), "11");
    TS_ASSERT_EQUALS(formatPlayerSetToNumberList(GPlayerSet(3)), "3");
    TS_ASSERT_EQUALS(formatPlayerSetToNumberList(GPlayerSet::allUpTo(11)), "1 2 3 4 5 6 7 8 9 10 11");
    TS_ASSERT_EQUALS(formatPlayerSetToNumberList(GPlayerSet()), "");
}

void
GamePlayerTestSuite::testMatrix()
{
    GPlayerBitMatrix mtx;

    // check zero-initialisation
    for (int i = 1; i <= 12; ++i)
        for (int j = 1; j <= 12; ++j)
            TS_ASSERT(!mtx.get(i, j));
    for (int i = 1; i <= 12; ++i)
        TS_ASSERT(mtx.getRow(i).empty());

    // set some value and check that surroundings are not modified. Do so twice.
    for (int i = 0; i < 2; ++i) {
        mtx.set(3, 7, true);
        TS_ASSERT(!mtx.get(2, 6));
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(2, 8));
        TS_ASSERT(!mtx.get(3, 6));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT(!mtx.get(3, 8));
        TS_ASSERT(!mtx.get(4, 6));
        TS_ASSERT(!mtx.get(4, 7));
        TS_ASSERT(!mtx.get(4, 8));
        TS_ASSERT(mtx.getRow(2).empty());
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7));
        TS_ASSERT(mtx.getRow(4).empty());
    }

    // set neighbouring bit (same row)
    for (int i = 0; i < 2; ++i) {
        mtx.set(3, 8, true);
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(2, 8));
        TS_ASSERT(!mtx.get(2, 9));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT( mtx.get(3, 8));
        TS_ASSERT(!mtx.get(3, 9));
        TS_ASSERT(!mtx.get(4, 7));
        TS_ASSERT(!mtx.get(4, 8));
        TS_ASSERT(!mtx.get(4, 9));
        TS_ASSERT(mtx.getRow(2).empty());
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7) | GPlayerSet(8));
        TS_ASSERT(mtx.getRow(4).empty());
    }

    // set neighbouring bit (different row)
    for (int i = 0; i < 2; ++i) {
        mtx.set(2, 6, true);
        TS_ASSERT(!mtx.get(1, 5));
        TS_ASSERT(!mtx.get(1, 6));
        TS_ASSERT(!mtx.get(1, 7));
        TS_ASSERT(!mtx.get(2, 5));
        TS_ASSERT( mtx.get(2, 6));
        TS_ASSERT(!mtx.get(2, 7));
        TS_ASSERT(!mtx.get(3, 5));
        TS_ASSERT(!mtx.get(3, 6));
        TS_ASSERT( mtx.get(3, 7));
        TS_ASSERT(mtx.getRow(1).empty());
        TS_ASSERT_EQUALS(mtx.getRow(2), GPlayerSet(6));
        TS_ASSERT_EQUALS(mtx.getRow(3), GPlayerSet(7) | GPlayerSet(8));
    }

    // check some out-of-range positions
    TS_ASSERT(!mtx.get(99, 2));
    TS_ASSERT(!mtx.get(99, 99));
    TS_ASSERT(!mtx.get(2, 99));
    TS_ASSERT(!mtx.get(-99, 99));
    TS_ASSERT(!mtx.get(1, 13));
    TS_ASSERT(!mtx.get(13, 1));

    // clear it again and check zeroness again
    mtx.clear();

    for (int i = 1; i <= 12; ++i)
        for (int j = 1; j <= 12; ++j)
            TS_ASSERT(!mtx.get(i, j));
    for (int i = 1; i <= 12; ++i)
        TS_ASSERT(mtx.getRow(i).empty());
}

void
GamePlayerTestSuite::testArray()
{
    GPlayerArray<int> n;

    // check indexing
    TS_ASSERT(!n.isValidIndex(-2));
    TS_ASSERT(!n.isValidIndex(-1));
    TS_ASSERT(n.isValidIndex(0));
    TS_ASSERT(n.isValidIndex(1));
    TS_ASSERT(n.isValidIndex(10));
    TS_ASSERT(n.isValidIndex(11));
    TS_ASSERT(n.isValidIndex(12));
    TS_ASSERT(!n.isValidIndex(-13));
    TS_ASSERT(!n.isValidIndex(-14));

    // check initialisation
    n.setAll(0);
    TS_ASSERT_EQUALS(n[0], 0);
    TS_ASSERT_EQUALS(n[1], 0);
    TS_ASSERT_EQUALS(n[2], 0);
    TS_ASSERT_EQUALS(n[10], 0);
    TS_ASSERT_EQUALS(n[11], 0);
    TS_ASSERT_EQUALS(n[12], 0);

    // check initialisation
    n.setAll(42);
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 42);
    TS_ASSERT_EQUALS(n[10], 42);
    TS_ASSERT_EQUALS(n[11], 42);
    TS_ASSERT_EQUALS(n[12], 42);

    // check assignment
    n[2] = 8;
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 8);
    TS_ASSERT_EQUALS(n[3], 42);
    TS_ASSERT_EQUALS(n[4], 42);

    // check modify-assignment
    n[2] += 7;
    TS_ASSERT_EQUALS(n[0], 42);
    TS_ASSERT_EQUALS(n[1], 42);
    TS_ASSERT_EQUALS(n[2], 15);
    TS_ASSERT_EQUALS(n[3], 42);
    TS_ASSERT_EQUALS(n[4], 42);
}

void
GamePlayerTestSuite::testRaceNameList()
{
    // Construct a race name list
    MemoryStream ms(string_t(file_racenm, file_racenm_size));
    GRaceNameList rnl(ms);

    // Check it
    TS_ASSERT_EQUALS(rnl.getShortName(1), "The Feds");
    TS_ASSERT_EQUALS(rnl.getFullName(1),  "The Solar Federation");
    TS_ASSERT_EQUALS(rnl.getAdjName(1),   "Fed");

    TS_ASSERT_EQUALS(rnl.getShortName(11), "The Colonies");
    TS_ASSERT_EQUALS(rnl.getFullName(11),  "The Missing Colonies of Man");
    TS_ASSERT_EQUALS(rnl.getAdjName(11),   "Colonial");

    // We cannot sensibly check index 0 and 12, because those are mangled
    // by NLS which is in an indeterminate state here. So we just check
    // that they don't throw or otherwise fail.
    rnl.getShortName(0);
    rnl.getFullName(0);
    rnl.getAdjName(0);
    rnl.getShortName(12);
    rnl.getFullName(12);
    rnl.getAdjName(12);

    // Expansions
    TS_ASSERT_EQUALS(rnl.expandNames("a %-5 ship"), "a Privateer ship");
    TS_ASSERT_EQUALS(rnl.expandNames("attack %5!"), "attack The Privateers!");
    TS_ASSERT_EQUALS(rnl.expandNames("%1..."), "The Feds...");
    TS_ASSERT_EQUALS(rnl.expandNames("%a..."), "The Rebels...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%b"), "...The Colonies");
    TS_ASSERT_EQUALS(rnl.expandNames("%A..."), "The Rebels...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%B"), "...The Colonies");
    TS_ASSERT_EQUALS(rnl.expandNames("%-A..."), "Rebel...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%-B"), "...Colonial");
    TS_ASSERT_EQUALS(rnl.expandNames("%1%2"), "The FedsThe Lizards");
    TS_ASSERT_EQUALS(rnl.expandNames("a%%b"), "a%b");
    TS_ASSERT_EQUALS(rnl.expandNames("%%"), "%");
    TS_ASSERT_EQUALS(rnl.expandNames("%%1"), "%1");

    // I'm not entirely sure about these. They are out of range. Right now,
    // they yield a blank result, but a non-blank result would be well within
    // the specification (such as a rule that '%' quotes the next character).
    TS_ASSERT_EQUALS(rnl.expandNames("%0"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%01"), "1");
    TS_ASSERT_EQUALS(rnl.expandNames("%c"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%x"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%."), "");
}
