/**
  *  \file t.io.dirs.cc
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "io/dirs.h"
#include "u/t_io.h"

/*  I'm somehow out of ideas how to test this.

    Directory:
      getEntryByName          DONE
      getEntries              DONE
      getParent
      getName
      openSubdir
      openFile
      openFileNT
      openFileT
      isAbsoluteName          (done)
      isSeparatorChar
      makeFileName
      getCanonicalName
      getAbsoluteName         (done)
      isAbsoluteName
      getFileName
      getDirName
      getWorkingDirectory
      openDir
      getRoots                DONE
    DirectoryEntry:
      openDir
      openParent              DONE
      openFile
      getName
      getBaseName             DONE
      getFileType
      getSize
      (getInfo)
      (setInfo) */

void
IoDirsTestSuite::testRoots()
{
    Ptr<Enumeration<Ptr<Directory> > > roots =
        Directory::getRoots();
    TS_ASSERT(roots);
    while (roots->hasMoreElements()) {
        Ptr<Directory> dir = roots->getNextElement();
        TS_ASSERT(dir);
    }
}

void
IoDirsTestSuite::testWorkingDirectory()
{
    string_t s = Directory::getWorkingDirectory();

    TS_ASSERT(Directory::isAbsoluteName(s));

    Ptr<Directory> dir = Directory::openDir(s);
    TS_ASSERT(dir);
    TS_ASSERT_EQUALS(Directory::getAbsoluteName(dir->getName()), s);

    Ptr<Enumeration<Ptr<DirectoryEntry> > > entries = dir->getEntries();
    TS_ASSERT(entries);
    while (entries->hasMoreElements()) {
        Ptr<DirectoryEntry> entry = entries->getNextElement();
        TS_ASSERT(entry);
        TS_ASSERT_EQUALS(Directory::getAbsoluteName(entry->openParent()->getName()), s);
        TS_ASSERT_EQUALS(dir->getEntryByName(entry->getBaseName())->getName(), entry->getName());
    }
}
